
# q-expansion of newform 1872.4.a.bf, downloaded from the LMFDB on 15 January 2025.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1872
weight = 4
poly_data = [-7, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0], 1], [[0, 2], 1]]

hecke_ring_character_values = None
aps_data = [[0, 0], [0, 0], [2, 1], [10, -3], [-30, -4], [-13, 0], [42, 6], [30, -17], [-36, 20], [62, 44], [54, 53], [18, 46], [26, -47], [16, -18], [-214, -70], [-190, -52], [-710, 34], [506, 8], [422, -7], [-434, 24], [-30, -154], [-136, 20], [-626, 122], [-286, 151], [354, 82], [-598, -150], [-1164, -50], [-432, -176], [1466, -92], [-1390, -20], [-412, -342], [32, -172], [1002, 33], [-804, 84], [-502, -137], [-686, 263], [1398, 212], [-2158, 177], [-1070, 410], [1066, 142], [-980, -704], [2478, -148], [-2136, 56], [-1374, -76], [-1262, 117], [-236, 250], [-524, -520], [-978, 577], [-1602, 808], [-1622, 412], [558, -566], [-1138, 32], [-4830, -386], [-2336, -304], [-1850, -738], [-428, 80], [-4434, 220], [4450, -339], [-6682, -232], [6234, 281], [216, -290], [594, 651], [7626, 391], [-6404, -292], [-2250, 1392], [218, 189], [5798, 773], [-1946, -1436], [2156, 452], [-2646, 786], [-2310, 425], [-5902, 914], [-1240, 1928], [-62, 248], [-5970, -453], [2934, -1734], [-2874, 42], [-4846, -1250], [7610, -253], [-6814, 450], [12336, -180], [1898, 1044], [3382, 2586], [-13322, -72], [-8260, -138], [6984, 768], [-1398, -2529], [-598, 190], [554, -2599], [3746, -607], [8664, -664], [-11334, -1764], [3846, -1339], [2280, -1320], [1014, 2109], [-13244, 596], [-16550, 681], [-5670, 1500], [3108, -2260], [7122, -2616], [3908, -124], [8786, -2457], [-3564, 3660], [-8170, -906], [6264, 1378], [3866, 4060], [-4266, -2598], [-3758, -3357], [-988, 2092], [-2090, -584], [9456, 2780], [10106, 2582], [15602, 889], [21698, -1065], [-10086, 591], [22746, -1426], [18518, 1785], [11720, -1468], [8598, -912], [-12612, 3764], [4074, -2846], [10850, -3796], [5362, -680], [7818, -3288], [-19594, 1677], [10870, -342], [17834, 1672], [16060, -3128], [6596, -3582], [-13158, -2056], [-23022, 259], [6606, -1736], [-24668, -2622], [11242, -1936], [14082, 2829], [970, -1872], [15154, 1339], [-24734, -2261], [26678, 1726], [-19782, -1644], [17174, 211], [15106, 1223], [-6412, 6114], [1354, -844], [13330, -3224], [-11278, -2200], [-11846, 4146], [22518, -1458], [-25812, 3524], [21150, -122], [-34870, 2058], [-20418, -2298], [-21584, 5062], [-10256, 3964], [8360, 2010], [-7184, -244], [3888, 7044], [-8758, 6225], [4662, -160], [5522, -7541], [-102, -3930], [11910, 1842], [-20938, -3359], [9564, 372], [11690, -7213], [-33878, 1868], [22792, -2524], [-2242, -5472], [36506, 2370], [25586, -4318], [9950, 2840], [5698, -4346], [-460, 8800], [-16254, -4878], [-26468, -1086], [-39858, 1824], [13410, 9623], [-19950, 6147], [42292, 1502], [51226, 1488], [9210, 4705], [-7524, -1784], [-25462, 2088], [-28294, 5055], [-19710, 2226], [-6202, -4750], [25158, -1772], [15342, -829], [27322, 2602], [-12558, 1338], [-18086, 428], [-3266, -354], [20364, -4072], [330, -2681], [49252, -5536], [16198, -4046], [-13862, 1724], [-56582, 536], [12674, -9841], [-5344, -7016], [42618, 697], [53896, -364], [-44838, 3512], [-28662, -10412], [1106, 9198], [-53566, -4624], [-36730, 5361], [-2168, -15604], [3114, -9945], [12208, -858], [-36010, -9960], [-6550, 424], [71392, -416], [9658, 3288], [-10602, -1866], [78770, -704], [-41344, -4448], [-69490, 3884], [-24282, -9984], [-67774, 3875], [-19106, 5216], [40178, 11085], [-2094, -4781], [-47866, 1533], [-4608, -1164], [84550, 3004], [21598, -12408], [16492, 7272], [36300, 1174], [-44918, 8082], [-86874, 564], [7316, 9280], [5558, 3215], [-7246, 2846], [70612, 5328], [31150, -4240], [25642, -15294], [-60294, 5943], [86300, 4736], [-57816, -5100], [34230, -3222], [12880, -14214], [18896, -17204], [62538, 2414], [-36774, -13341], [-80084, 6604], [-66358, 865], [6626, -10502], [470, 1399], [-21284, 16952], [25170, 7504], [55242, 4761], [41642, -6044], [11054, 17532], [20046, 14376], [40970, -6720], [-46622, 1208], [51302, 7535], [71542, -8638], [10946, 3320], [-74276, 10302], [-27300, 17244], [19042, 6964], [-56650, 5880], [40458, -15313], [34380, 1816], [-10022, -5051], [82938, -1191], [-70310, -2213], [-47142, -4374], [4218, -4412], [-24946, 2349], [-10630, 4286], [-98660, -346], [-65282, 6096], [-8066, -9925], [91494, 9156], [100954, 1198], [77818, -11068], [50488, -15008], [24504, 4488], [-111534, 4505], [-95104, -2080], [-1166, -798], [-3334, 14071], [30264, 13140], [-52986, 9424], [-30278, 4583], [-85078, 5231], [43918, 2698], [55098, -7968], [5668, 17044], [122714, 4579], [-89230, 8962], [-31418, -23340], [133806, -2682], [21536, -7872], [9958, 19954], [6964, -22396], [60522, 287], [3994, 14072], [-52590, -9999], [25972, -182], [-41492, -4352], [-79900, -3432], [-74134, 2478], [-94676, -7900], [19318, 23096], [20634, 8868], [107458, -1812], [-71076, -5640], [850, -2347], [-32438, 9748], [-12052, 620], [9362, -9914], [-25786, -20648], [97910, 6056], [-47378, -5846], [-33038, 7526], [-67966, 14174], [-12336, -9218], [2546, -30380], [-33070, 26156], [-125126, 9519], [94650, -4881], [12182, -200], [-4494, 1409], [68718, 489], [21992, 11276], [-89802, 19160], [168074, 2320], [9230, -25384], [39176, -12624], [-9062, 6016], [26958, -22349], [-3362, 20046], [107914, 7630], [-112046, -18191], [10410, -27554], [75116, 19038], [101354, -318], [-100638, 11088], [-1966, 32179], [31396, -2054], [530, 3873], [-4972, -20660], [-12542, -37632], [3842, -2659], [-89862, -1326], [-104930, 7910], [61286, -13943], [-37982, 27054], [-68262, -2221], [65340, 11206], [20018, -21708], [50978, 6220], [-51086, 19146], [15686, -22892], [-57034, 6354], [66430, -20790], [-23614, 24042], [-31694, -25406], [32308, 29188], [-57882, -32868], [94312, 25342], [-124070, -17868], [30122, 29077], [35018, -781], [77770, 20004], [31672, 21224], [22624, -14314], [237866, 1374], [21530, -37676], [82896, -9644], [32710, 29184], [44718, -27474], [-18640, -6908], [-46310, -3500], [74286, 9036], [-5118, 8600], [102306, 1479], [60970, -24723], [158274, -10975], [-57686, 30127], [155186, -20823], [-131422, -20300], [72030, -21873], [-168390, -8976], [-102106, 9235], [-78412, 9988], [-15782, -48782], [-200214, -4401], [-18198, 32380], [134436, -1460], [89242, -24286], [-116450, -18320], [123214, -4065], [155202, -16638], [1804, 22668], [-13782, 10437], [-24542, -13174], [-116766, -31330], [107866, 4829], [107754, -11517], [1434, 13149], [16568, 14116], [94202, -10680], [-189894, -3541], [-62382, -36217], [4498, 10056], [176482, -17464], [-718, 26232], [29308, -25028], [89056, 16514], [93690, 22452], [-113526, -5840], [26886, -41168], [-58072, -29180], [-8734, -12373], [32940, 25296], [125802, 3866], [-12510, 33274], [108750, 11556], [-46608, 3228], [-20150, -37972], [214618, 7075], [156176, 12880], [108074, -2589], [-60846, 28717], [-57792, 38988]]
